#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Analysis {

  [StorableClass]
  public class SnapshotCreaterAnalyzer<T> : InstrumentedOperator, IAnalyzer where T : struct {

    #region Parameter properties
    public ScopeTreeLookupParameter<ValueTypeArray<T>> ArrayParameter {
      get { return (ScopeTreeLookupParameter<ValueTypeArray<T>>)Parameters["Array"]; }
    }
    #endregion

    #region Properties
    public bool EnabledByDefault {
      get { return true; }
    }
    #endregion

    #region Storage & Cloning
    [StorableConstructor]
    protected SnapshotCreaterAnalyzer(bool deserializing) : base(deserializing) { }
    protected SnapshotCreaterAnalyzer(SnapshotCreaterAnalyzer<T> original, Cloner cloner) : base(original, cloner) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new SnapshotCreaterAnalyzer<T>(this, cloner);
    }
    #endregion

    public SnapshotCreaterAnalyzer() {
      Parameters.Add(new ScopeTreeLookupParameter<ValueTypeArray<T>>("Array", "The array of values that should get a new snapshot."));
      ArrayParameter.Hidden = false;
    }

    public override IOperation InstrumentedApply() {
      var arrays = ArrayParameter.ActualValue;
      foreach (var a in arrays) {
        a.CreateSnapshot();
      }
      return base.InstrumentedApply();
    }
  }
}
